#ifndef __BSCWINDOW_H
#define __BSCWINDOW_H

#include <DirectWindow.h>
#include <Locker.h>
#include <MediaDefs.h>
#include <MediaFormats.h>
#include <Path.h>

#include <queue>

struct capture_options
{
	BRect bounds;
	bool includeCursor;
	BPath fileName;
	uint32 framesPerSecond;
	
	media_format_family formatFamily;
	media_format format;
	media_codec_info codecInfo;
	
	bool useDirectWindow;
	
	bool notifyStatusBar;
};


class BButton;
class BStatusBar;
class BStopWatch;
class BStringView;
class BTabView;
class BSCWindow : public BDirectWindow {
public:
	BSCWindow();
	virtual bool QuitRequested();
	virtual void MessageReceived(BMessage *);
	virtual void ScreenChanged(BRect screen_size, color_space depth);
	virtual void DirectConnected(direct_buffer_info *info);
	
	status_t GetBitmap(BBitmap **bitmap, BRect bounds);
	status_t ReadBitmap(BBitmap *bitmap, BRect bounds);
	
private:
	BTabView *fTabView;
	BButton *fStartStopButton;
	BStringView *fStringView;
	BStringView *fRecordingInfo;
	BStatusBar *fStatusBar;
	
	direct_buffer_info fDirectInfo;
	//BLocker fDirectLock;
	
	BStopWatch *fStopWatch;
	thread_id fCaptureThread;
	thread_id fSaverThread;
	bool fKillThread;
	
	int32 fBitmapCount;
	BLocker fLocker;
	queue<BBitmap *> *fBitmaps;
	queue<BPoint> *fCursorPositions;
	capture_options fCaptureOptions;
		
	void StartCapture();
	void EndCapture();
	
	static int CaptureThread(void *arg);
	static int SaverThread(void *arg);
};

#endif
