#include "RectUtils.h"
#include "SelectionWindow.h"

#include <Bitmap.h>
#include <Message.h>
#include <Screen.h>
#include <StringView.h>
//#include <View.h>

#include <cstdio>

const char *kInfo = "Select the area to capture";

class TSelectionView : public BView {
public:
	TSelectionView(BRect frame, const char *name);
	virtual void MouseDown(BPoint where);
	virtual void MouseUp(BPoint where);
	virtual void MouseMoved(BPoint where, uint32 code, const BMessage *a_message);
	virtual void Draw(BRect updateRect);
	
	BRect SelectionRect();
		
private:
	bool fMouseDown;
	BPoint fSelectionStart;
	BPoint fSelectionEnd;
	
	BRect fStringRect;
	
	void MakeSelectionRect(BRect *rect);
};


TSelectionView::TSelectionView(BRect frame, const char *name)
	:
	BView(frame, name, B_FOLLOW_NONE, B_WILL_DRAW),
	fMouseDown(false)
{
	SetFontSize(20);
	
	float stringWidth = StringWidth(kInfo);
	font_height height;
	GetFontHeight(&height);
	float stringHeight = height.ascent + height.descent + height.leading;
	
	fStringRect.left = (Bounds().Width() - stringWidth) / 2;
	fStringRect.top = (Bounds().Height() - stringHeight) / 2;
	fStringRect.right = fStringRect.left + stringWidth;
	fStringRect.bottom = fStringRect.top + stringHeight;
}


void
TSelectionView::MouseDown(BPoint where)
{
	fMouseDown = true;
	fSelectionStart = where;
}


void
TSelectionView::MouseMoved(BPoint where, uint32 code, const BMessage *a_message)
{
	if (fMouseDown && code == B_INSIDE_VIEW) {
		BRect selectionRect;
		MakeSelectionRect(&selectionRect);
	
		BRect newSelection;	
		fSelectionEnd = where;
		MakeSelectionRect(&newSelection);
		
		Invalidate(selectionRect | newSelection);
	}
}


void
TSelectionView::MouseUp(BPoint where)
{
	fSelectionEnd = where;
	fMouseDown = false;
	
	Window()->PostMessage(B_QUIT_REQUESTED);	
}


void
TSelectionView::Draw(BRect updateRect)
{
	BView::Draw(updateRect);
	PushState();
	SetHighColor(0, 0, 0);
	SetDrawingMode(B_OP_INVERT);
	//if (updateRect.Intersects(fStringRect.InsetByCopy(-30, -30)))
	//	DrawString(kInfo, fStringRect.LeftTop());
	
	if (fMouseDown) {
		BRect selection;
		MakeSelectionRect(&selection);
		StrokeRect(selection, B_MIXED_COLORS);
	}
	PopState();
}


BRect
TSelectionView::SelectionRect()
{
	BRect rect;
	MakeSelectionRect(&rect);
	return rect;
}


void
TSelectionView::MakeSelectionRect(BRect *rect)
{
	rect->SetLeftTop(BPoint(min_c(fSelectionStart.x, fSelectionEnd.x),
					min_c(fSelectionStart.y, fSelectionEnd.y)));
	rect->SetRightBottom(BPoint(max_c(fSelectionStart.x, fSelectionEnd.x),
					max_c(fSelectionStart.y, fSelectionEnd.y)));
}


TSelectionWindow::TSelectionWindow()
	:
	BWindow(BScreen().Frame(), "Area Window", B_BORDERED_WINDOW,
		B_ASYNCHRONOUS_CONTROLS|B_NOT_RESIZABLE|B_NOT_CLOSABLE|B_NOT_ZOOMABLE)
{
	fView = new TSelectionView(Bounds(), "Selection view");
	AddChild(fView);
}


void
TSelectionWindow::Show()
{
	snooze(30000);
	
	BBitmap *bitmap = NULL;	
	BScreen(this).GetBitmap(&bitmap, false);
	fView->SetViewBitmap(bitmap);
	delete bitmap;
	
	BWindow::Show();	
}


void
TSelectionWindow::SetTarget(BLooper *target)
{
	fTarget = target;
}


void
TSelectionWindow::SetCommand(uint32 command)
{
	fCommand = command;	
}


bool
TSelectionWindow::QuitRequested()
{
	BMessage message(fCommand);
	message.AddRect("selection", fView->SelectionRect());
	fTarget->PostMessage(&message);
	
	return BWindow::QuitRequested();
}
